import {
  getCircleRadius,
  getContinentColor,
  gdpScale,
  lifeScale,
  tooltip,
  factBox,
} from './setup.js';

export function popUpTooltip(event, data, year) {
  tooltip
    .style('opacity', 1)
    .style('z-index', 10)
    .html(
      `<strong style="font-size: 1.2em;">${data.country}</strong><br/>
      Pop (millions): ${Math.ceil(data.population.get(year) / 1000000)}<br/>
      GDP per capita: ${data.income.get(year)}<br/>
      Mean lifespan: ${data.life_exp.get(year)}`,
    );
}

export function adjustTooltip(event) {
  tooltip
    .style('left', event.x - 160 + 'px')
    .style('top', event.y - 120 + 'px');
}

export function hideTooltip() {
  tooltip.style('opacity', 0).style('z-index', -1);
}

export function highlightMarks(obj, data, year) {
  let continent = d3.select(obj).attr('class');

  // Select just the country scatter point (that triggered the event)
  d3.select(obj)
    .attr('fill', 'red')
    .attr('opacity', 1.0)
    .attr('r', () => getCircleRadius(data, year) * 1.5);

  // Select *every* line
  d3.selectAll(`path.line`).attr('opacity', 0.0);

  // Select all continent lines
  d3.selectAll(`path.${continent}.line`).attr(
    'opacity',
    0.2,
  );

  // Select just the country line; I'm doing this after I handle all the
  // continent lines so I overwrite specific properties for just the country
  d3.select(`path#${obj.id}`)
    .attr('stroke', 'black')
    .attr('opacity', 1.0)
    .attr('stroke-width', 3);
}

export function resetMarks(obj, data, year) {
  let continent = d3.select(obj).attr('class');

  // Select just the country scatter point (that triggered the event)
  d3.select(obj)
    .attr('fill', getContinentColor(data))
    .attr('opacity', 0.7)
    .attr('r', () => getCircleRadius(data, year));

  d3.selectAll('path.line').attr('opacity', 0.2);

  // Select just the country line
  d3.selectAll(`path#${obj.id}`)
    .attr('stroke', getContinentColor(data))
    .attr('stroke-width', 1.5);
}

export function setFactBox(
  fact,
  data,
  curYear,
  curContinent,
) {
  factBox.html(fact.text);
  if (fact.country) {
    let gdp = data.get(fact.country).income.get(curYear);
    let life = data.get(fact.country).life_exp.get(curYear);
    factBox
      .style('left', gdpScale(gdp) + 'px')
      .style('top', lifeScale(life) + 'px');
  } else {
    factBox.style('left', '500px').style('top', '0px');
  }
}
