import {
  svg1,
  svg2,
  slider,
  dropdown,
  gdpScale,
  lifeScale,
  yearScale,
  radiusScale,
  colorScale,
  getCircleRadius,
  getContinentColor,
  lineGenerator,
  factBox,
} from './setup.js';

import {
  highlightMarks,
  resetMarks,
  popUpTooltip,
  adjustTooltip,
  hideTooltip,
  setFactBox,
} from './events.js';

import { facts } from './annotations.js';

import { processRawData } from './preprocess.js';

var data = new Map();
var curYear = '1800';
var curContinent = 'global';
var validCountries = new Array();

slider.on('input', function () {
  curYear = this.value;
  if (facts.has(curYear)) {
    let fact = facts.get(curYear);
    setFactBox(fact, data, curYear, curContinent);
    factBox.transition().duration(200).style('opacity', 1);
  } else {
    factBox.transition().duration(1000).style('opacity', 0);
  }
  d3.select('#yearLabel').text(`Year: ${curYear}`);
  d3.select('#sliderLabel').text(`Year: ${curYear}`);
  renderData(300);
});

console.log(dropdown);
dropdown.on('change', function () {
  curContinent = d3.select(this).property('value');
  renderData(0);
});

function renderData(duration) {
  validCountries = [...data.values()].filter(
    (c) =>
      c.income.get(curYear) > 0 &&
      c.life_exp.get(curYear) > 0 &&
      (curContinent === 'global' ||
        c.continent === curContinent),
  );

  const circleMarks = svg1
    .selectAll('circle')
    .data(validCountries)
    .join('circle')
    .attr('r', 3)
    .attr('stroke', 'black')
    .attr('r', (d) => getCircleRadius(d, curYear))
    .attr('opacity', 0.7)
    .attr('fill', getContinentColor)
    .attr('class', (d) => d.continent)
    .attr('id', (d) => d.country)
    .on('mouseenter', function (event, data) {
      highlightMarks(this, data, curYear);
      popUpTooltip(event, data, curYear);
    })
    .on('mousemove', function (event, data) {
      adjustTooltip(event);
    })
    .on('mouseleave', function (event, data) {
      resetMarks(this, data, curYear);
      hideTooltip();
    })
    .transition()
    .duration(duration)
    .attr('cx', (d) => gdpScale(d.income.get(curYear)))
    .attr('cy', (d) => lifeScale(d.life_exp.get(curYear)));

  const lineMarks = svg2
    .selectAll('path.line')
    .data(validCountries)
    .join('path')
    .attr('opacity', 0.1)
    .attr('fill', 'none')
    .attr('stroke-width', 1.5)
    .attr('stroke', getContinentColor)
    .attr('class', (d) => d.continent)
    .classed('line', true)
    .attr('id', (d) => d.country)
    .attr('d', (d) => lineGenerator(d.life_exp.entries()));

  annotatePlot();
}

function annotatePlot() {
  //svg2.selectAll('#yearMarkerLine').remove();
  svg2
    .selectAll('#yearMarkerLine')
    .data([
      [
        [curYear, 0],
        [curYear, 90],
      ],
    ])
    .join('path')
    .attr('opacity', 0.8)
    .attr('fill', 'none')
    .attr('stroke-width', 1.5)
    .attr('stroke', 'black')
    .attr('id', 'yearMarkerLine')
    .attr('d', (d) => lineGenerator(d));

  let globalAverage = d3.mean(
    d3.map(validCountries, (c) => c.life_exp.get(curYear)),
  );
  svg2
    .selectAll('#globalAverageMark')
    .data([null])
    .join('circle')
    .attr('r', 5)
    .attr('fill', 'none')
    .attr('stroke', 'black')
    .attr('stroke-width', 2)
    .attr('cx', yearScale(curYear))
    .attr('cy', lifeScale(globalAverage))
    .attr('id', 'globalAverageMark');
}

//Read the data
const countryStatsUrl =
  'https://raw.githubusercontent.com/adamjanes/udemy-d3/master/05/5.10.1/data/data.json';
d3.json(countryStatsUrl).then((rawData) => {
  data = processRawData(rawData);
  console.log(data);
  renderData(0);
});
